@@ -4,13 +4,14 @@
 Python FIGlet adaption
 """
 
-import os
+from __future__ import print_function
 import pkg_resources
 import re
 import sys
 from optparse import OptionParser
 
-__version__ = '0.6.1dev'
+from .version import __version__
+
 __author__ = 'Peter Waller <peter.waller@gmail.com>'
 __copyright__ = """
 Copyright (C) 2007 Christopher Jones <cjones@gruntle.org>
@@ -32,15 +33,16 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 """
 
+
 DEFAULT_FONT='standard'
 
 
 def figlet_format(text, font=DEFAULT_FONT, **kwargs):
-    fig = Figlet(font)
-    return fig.renderText(text, **kwargs)
+    fig = Figlet(font, **kwargs)
+    return fig.renderText(text)
 
 def print_figlet(text, font=DEFAULT_FONT, **kwargs):
-    print figlet_format(text, font, **kwargs)
+    print(figlet_format(text, font, **kwargs))
 
 
 class FigletError(Exception):
@@ -78,25 +80,47 @@ class FigletFont(object):
         self.comment = ''
         self.chars = {}
         self.width = {}
-        self.data = None
-
-        for extension in ('tlf', 'flf'):
-            fn = '%s.%s' % (font, extension)
-            if pkg_resources.resource_exists('pyfiglet.fonts', fn):
-                self.data = pkg_resources.resource_string('pyfiglet.fonts', fn)
-                break
-        else:
-            raise FontNotFound(font)
-
+        self.data = self.preloadFont(font)
         self.loadFont()
 
     @classmethod
-    def getFonts(self):
+    def preloadFont(cls, font):
+        """
+        Load font data if exist
+        """
+        for extension in ('tlf', 'flf'):
+            fn = '%s.%s' % (font, extension)
+            if pkg_resources.resource_exists('pyfiglet.fonts', fn):
+                data = pkg_resources.resource_string('pyfiglet.fonts', fn)
+                data = data.decode('ascii', 'replace')
+                return data
+        else:
+            raise FontNotFound(font)
+
+    @classmethod
+    def getFonts(cls):
         return [font.rsplit('.', 2)[0] for font
                 in pkg_resources.resource_listdir('pyfiglet', 'fonts')
                 if font.endswith(('.flf', '.tlf'))
-                   and self.reMagicNumber.search(pkg_resources.resource_stream(
-                        'pyfiglet.fonts', font).readline())]
+                   and cls.reMagicNumber.search(pkg_resources.resource_stream(
+                        'pyfiglet.fonts', font).readline().decode('ascii', 'replace'))]
+
+    @classmethod
+    def infoFont(cls, font, short=False):
+        """
+        Get informations of font
+        """
+        data = FigletFont.preloadFont(font)
+        infos = []
+        reMagicNumber = re.compile(r'^[tf]lf2.')
+        reStartMarker = re.compile(r'^(FONT|COMMENT|FONTNAME_REGISTRY|FAMILY_NAME|FOUNDRY|WEIGHT_NAME|SETWIDTH_NAME|SLANT|ADD_STYLE_NAME|PIXEL_SIZE|POINT_SIZE|RESOLUTION_X|RESOLUTION_Y|SPACING|AVERAGE_WIDTH|COMMENT|FONT_DESCENT|FONT_ASCENT|CAP_HEIGHT|X_HEIGHT|FACE_NAME|FULL_NAME|COPYRIGHT|_DEC_|DEFAULT_CHAR|NOTICE|RELATIVE_).*')
+        reEndMarker = re.compile(r'^.*[@#$]$')
+        for line in  data.splitlines()[0:100]:
+            if reMagicNumber.search(line) is None \
+               and reStartMarker.search(line) is None \
+               and reEndMarker.search(line) is None:
+                infos.append(line)
+        return '\n'.join(infos) if not short else infos[0]
 
     def loadFont(self):
         """
@@ -112,18 +136,17 @@ class FigletFont(object):
 
             header = self.reMagicNumber.sub('', header)
             header = header.split()
-            
+
             if len(header) < 6:
                 raise FontError('malformed header for %s' % self.font)
 
             hardBlank = header[0]
             height, baseLine, maxLength, oldLayout, commentLines = map(int, header[1:6])
-            printDirection = fullLayout = codeTagCount = None
+            printDirection = fullLayout = None
 
             # these are all optional for backwards compat
             if len(header) > 6: printDirection = int(header[6])
             if len(header) > 7: fullLayout = int(header[7])
-            if len(header) > 8: codeTagCount = int(header[8])
 
             # if the new layout style isn't available,
             # convert old layout style. backwards compatability
@@ -146,8 +169,11 @@ class FigletFont(object):
             for i in range(0, commentLines):
                 self.comment += data.pop(0)
 
-            # Load characters
-            for i in range(32, 127):
+            def __char(data):
+                """
+                Function loads one character in the internal array from font
+                file content
+                """
                 end = None
                 width = 0
                 chars = []
@@ -161,11 +187,29 @@ class FigletFont(object):
 
                     if len(line) > width: width = len(line)
                     chars.append(line)
+                return width, chars
 
-                if ''.join(chars) != '':
-                    self.chars[i] = chars
+            # Load ASCII standard character set (32 - 127)
+            for i in range(32, 127):
+                width, letter = __char(data)
+                if ''.join(letter) != '':
+                    self.chars[i] = letter
                     self.width[i] = width
 
+            # Load ASCII extended character set
+            while data:
+                line = data.pop(0).strip()
+                i = line.split(' ', 1)[0]
+                if (i == ''):
+                    continue
+                hex_match = re.search('^0x', i, re.IGNORECASE)
+                if hex_match is not None:
+                    i = int(i, 16)
+                    width, letter = __char(data)
+                    if ''.join(letter) != '':
+                        self.chars[i] = letter
+                        self.width[i] = width
+
         except Exception as e:
             raise FontError('problem parsing %s font: %s' % (self.font, e))
 
@@ -304,11 +348,11 @@ class FigletRenderingEngine(object):
             lineRight = curChar[row]
             if self.base.direction == 'right-to-left':
                 lineLeft, lineRight = lineRight, lineLeft
-            
+
             linebd = len(lineLeft.rstrip()) - 1
             if linebd < 0:
                 linebd = 0
-                
+
             if linebd < len(lineLeft):
                 ch1 = lineLeft[linebd]
             else:
@@ -342,7 +386,7 @@ class FigletRenderingEngine(object):
         buffer = ['' for i in range(self.base.Font.height)]
 
         for c in map(ord, list(text)):
-            if self.base.Font.chars.has_key(c) is False: continue
+            if not c in self.base.Font.chars: continue
             curChar = self.base.Font.chars[c]
             self.curCharWidth = self.base.Font.width[c]
             maxSmush = self.smushAmount(buffer=buffer, curChar=curChar)
@@ -410,9 +454,9 @@ class Figlet(object):
         self.engine = FigletRenderingEngine(base=self)
 
     def setFont(self, **kwargs):
-        if kwargs.has_key('font'):
+        if 'font' in kwargs:
             self.font = kwargs['font']
-        
+
         self.Font = FigletFont(font=self.font)
 
     def getDirection(self):
@@ -451,7 +495,7 @@ class Figlet(object):
 
 
 def main():
-    parser = OptionParser(version=__version__, usage='%prog [options] text..')
+    parser = OptionParser(version=__version__, usage='%prog [options] [text..]')
     parser.add_option('-f', '--font', default=DEFAULT_FONT,
             help='font to render with (default: %default)', metavar='FONT')
     parser.add_option('-D', '--direction', type='choice', choices=('auto', 'left-to-right', 'right-to-left'),
@@ -462,8 +506,18 @@ def main():
             help='set terminal width for wrapping/justification (default: %default)' )
     parser.add_option('-r', '--reverse', action='store_true', default=False, help='shows mirror image of output text')
     parser.add_option('-F', '--flip', action='store_true', default=False, help='flips rendered output text over')
+    parser.add_option('-l', '--list_fonts', action='store_true', default=False, help='show installed fonts list')
+    parser.add_option('-i', '--info_font', action='store_true', default=False, help='show font\'s information, use with -f FONT')
     opts, args = parser.parse_args()
 
+    if opts.list_fonts:
+        print('\n'.join(sorted(FigletFont.getFonts())))
+        exit(0)
+
+    if opts.info_font:
+        print(FigletFont.infoFont(opts.font))
+        exit(0)
+
     if len(args) == 0:
         parser.print_help()
         return 1
@@ -478,7 +532,7 @@ def main():
     r = f.renderText(text)
     if opts.reverse is True: r = r.reverse()
     if opts.flip is True: r = r.flip()
-    print r
+    print(r)
 
     return 0
 
